#define SN "DimmableLED & SwitchCtrl"
#define SV "1.0"

#include <MySensor.h>
#include <SPI.h>
#include <Bounce2.h>
#define FADE_DELAY 10  // Delay in ms for each percentage fade up/down (10ms = 1s full-range dim)

MySensor gw;

#include "dSensor.h"

void setup()
{
#ifdef DEBUG
  Serial.println( SN );
#endif

  gw.begin( incomingMessage );
  gw.sendSketchInfo(SN, SV);

  // Перебор и регистрация всех PWM портов
  for (int i = 0; i < numChannels; i++) {
    // Register the LED Dimmable Light with the gateway
    gw.present( i, S_DIMMER );

    // Pull the gateway's current dim level - restore light level upon sendor node power-up
    gw.request( i, V_DIMMER );
  }
#ifdef DEBUG
  Serial.print( "channels " );
  Serial.println ( numChannels );
#endif

  // Перебор и регистрация всех портов ввода (выключателей)
  for (int i = 0; i < numSwitches; i++) {
    // Setup the button
    pinMode(switches[i].dataPin, INPUT);
    // Activate internal pull-up
    digitalWrite(switches[i].dataPin, HIGH);

    // After setting up the button, setup debouncer
    switches[i].debouncer.attach(switches[i].dataPin);
    switches[i].debouncer.interval(5);

    // Register binary input sensor to gw (they will be created as child devices)
    // You can use S_DOOR, S_MOTION or S_LIGHT here depending on your usage.
    // If S_LIGHT is used, remember to update variable type you send in. See "msg" above.
    gw.present(switches[i].childId, S_DOOR);
  }
}

void loop()
{
  gw.process();
  for (int i = 0; i < numSwitches; i++) {
    switches[i].debouncer.update();
    // Get the update value
    int value = switches[i].debouncer.read();

    if (value != switches[i].oldVal) {

      int newLvl = registers[i].mem == 1? registers[i].memLvl : 100;
      int lvl = value == 1 ? 0 : newLvl; // Лог 1 на привязанном пине соответсвует состоянию Выкл на привязанном канале
      fadeToLevel( i, lvl );
#ifdef DEBUG
      Serial.print( "Switch " );
      Serial.print( i );
      Serial.print( " change state to " );
      Serial.print( value );
      Serial.print( " parent channel fade to  " );
      Serial.print( lvl );
      Serial.println( "" );
#endif

      // Send in the new value
      gw.send(switches[i].msg.set(value == HIGH ? 1 : 0));
      gw.send(registers[i].lightMsg.set(value == HIGH ? 0 : 1));
      gw.send(registers[i].dimmerMsg.set(lvl));
      switches[i].oldVal = value;
    }
  }

}

void incomingMessage(const MyMessage &message) {
  int si = message.sensor;
#ifdef DEBUG
  Serial.print( "message " );
  Serial.print( si );
  Serial.println( "" );
#endif

  if (message.type == V_LIGHT || message.type == V_DIMMER) {

    //  Retrieve the power or dim level from the incoming request message
    int requestedLevel = atoi( message.data );

    // Adjust incoming level if this is a V_LIGHT variable update [0 == off, 1 == on]
    /*
    requestedLevel *= ( message.type == V_LIGHT ? 100 : 1 );
    requestedLevel = constrain(requestedLevel, 0, 100);
    */
    if (message.type == V_LIGHT){
        requestedLevel = requestedLevel <=0 ? 0 : registers[si].mem == 1 ? registers[si].memLvl : requestedLevel;
    }
    requestedLevel = constrain(requestedLevel, 0, 100);


    // запишем актуальное значение яркости для эффекта Память
    if ( message.type == V_DIMMER && requestedLevel > 0 ) {
      registers[si].memLvl = requestedLevel;
    }

#ifdef DEBUG
    Serial.print( "Changing level to " );
    Serial.print( requestedLevel );
    Serial.print( ", from " );
    Serial.println( registers[si].currLvl );
#endif

    fadeToLevel( si, requestedLevel );

    // Inform the gateway of the current DimmableLED's SwitchPower1 and LoadLevelStatus value...
    gw.send(registers[si].lightMsg.set(registers[si].currLvl > 0 ? 1 : 0));

    // hek comment: Is this really nessesary?
    gw.send( registers[si].dimmerMsg.set(registers[si].currLvl) );


  }
}

/***
    This method provides a graceful fade up/down effect
*/
void fadeToLevel( int si, int toLevel ) {

  int delta = ( toLevel - registers[si].currLvl ) < 0 ? -1 : 1;

  while ( registers[si].currLvl != toLevel ) {
    registers[si].currLvl += delta;
    analogWrite( registers[si].ledPin, (int)(registers[si].currLvl / 100. * 255) );
    delay( FADE_DELAY );
  }
}
